"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var electron_1 = require("electron");
var Singleton_1 = __importDefault(require("./base/Singleton"));
var EventEnum_1 = require("./EventEnum");
var lodash_1 = __importDefault(require("lodash"));
/**
 * 是否是主线程
 */
var IS_MAIN = electron_1.ipcMain !== undefined;
var EventManager = /** @class */ (function (_super) {
    __extends(EventManager, _super);
    function EventManager() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._listener = new Map();
        _this._windows = [];
        return _this;
    }
    EventManager.prototype.mainInit = function (windows) {
        var _this = this;
        this._windows = windows;
        electron_1.ipcMain.on(EventEnum_1.MainEventType.ExecuteOtherWindowsListener, function (_event, type) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            _this._executeMain.apply(_this, __spreadArray([type], args, false));
        });
    };
    EventManager.prototype.rendererInit = function () {
        var _this = this;
        electron_1.ipcRenderer.on(EventEnum_1.MainEventType.ExecuteOtherWindowsListener, function (_, _senderType, type) {
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            console.log('渲染线程接收 Send', args);
            _this._execute.apply(_this, __spreadArray([type, true], args, false));
        });
    };
    EventManager.prototype._addEventListener = function (type, cb) {
        console.log(IS_MAIN ? '主线程添加监听器' : '渲染线程添加监听器');
        if (!this._listener.has(type)) {
            this._listener.set(type, new Array());
        }
        this._listener.get(type).push(cb);
        // 添加进程对应的事件
        if (IS_MAIN) {
            electron_1.ipcMain.on(type, function (_) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                cb.apply(void 0, args);
            });
        }
        else {
            electron_1.ipcRenderer.on(type, function (_) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                cb.apply(void 0, args);
            });
        }
    };
    EventManager.prototype._removeEventListener = function (type, cb) {
        if (!this._listener.has(type)) {
            throw new Error('没有事件类型');
        }
        var cbs = this._listener.get(type);
        if (!cbs) {
            return;
        }
        var index = cbs.indexOf(cb);
        if (~index) {
            cbs.splice(index, 1);
            this._listener.set(type, cbs);
            (IS_MAIN ? electron_1.ipcMain : electron_1.ipcRenderer).off(type, cb);
        }
    };
    EventManager.prototype.addEventListener = function (type, cb) {
        this._addEventListener(type, cb);
    };
    EventManager.prototype.removeEventListener = function (type, cb) {
        this._removeEventListener(type, cb);
    };
    EventManager.prototype.broadcast = function (type, arg1, arg2, arg3, arg4, arg5) {
        var args = [arg1, arg2, arg3, arg4, arg5].filter(function (arg) { return arg !== undefined; });
        // 执行自己的监听器
        this._execute.apply(this, __spreadArray([type, false], args, false));
        // 是主线程则向其他渲染线程发送消息
        if (IS_MAIN) {
            this._executeMain.apply(this, __spreadArray([type], args, false));
        }
        else {
            // 是渲染线程则让渲染线程通过主线程转发到其他渲染线程
            electron_1.ipcRenderer.send.apply(electron_1.ipcRenderer, __spreadArray([EventEnum_1.MainEventType.ExecuteOtherWindowsListener, type], args, false));
        }
    };
    EventManager.prototype._execute = function (type, isMainSend) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        console.log(type, 'execute');
        var callback = this._listener.get(type);
        if (callback) {
            callback.forEach(function (cb) {
                cb.apply(void 0, args);
            });
        }
        // 不是主线程 && 不是主线程调用的广播
        if (!IS_MAIN && !isMainSend) {
            electron_1.ipcRenderer.send.apply(electron_1.ipcRenderer, __spreadArray([type], args, false));
        }
    };
    EventManager.prototype._executeMain = function (type) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        console.log(type, 'executeMain');
        lodash_1.default.forEach(this._windows, function (item) {
            var _a;
            (_a = item.window.webContents).send.apply(_a, __spreadArray([EventEnum_1.MainEventType.ExecuteOtherWindowsListener,
                item.type,
                type], args, false));
        });
    };
    return EventManager;
}(Singleton_1.default));
exports.default = EventManager;
